from flet import *


class Dock(Container):
    def __init__(
        self,
        dock_icon: str = icons.MAIL,
        icon_color: str = "white",
        dock_color: str = "red",
        count: int = 4,
        max_value: int = 9,
        position: str = "top_left",
    ):
        super().__init__()
        self.max_value = max_value
        self.dock_color = dock_color
        self.content = Stack(
            [
                Container(
                    Icon(dock_icon, color=icon_color, size=30),
                    height=50,
                    width=60,
                    margin=margin.only(left=-16 if "left" in position else 5, right=15),
                ),
                self.create_dock(count, position),
            ]
        )

        self.alignment = alignment.center

    def create_dock(self, count: int, position: str):
        display_count = f"{count}" if count <= self.max_value else f"{self.max_value}+"

        # Adjasting the top container size based on whether a single,2 or 3 digits number are given
        dock_width = 20  # default width for single-digit numbers
        if self.max_value >= 10:
            dock_width = 25  # width for two-digit numbers
        if self.max_value >= 100:
            dock_width = 30  # width for three-digit numbers

        dock = Container(
            Text(
                display_count,
                color="white",
                weight=FontWeight.W_500,
                size=11,
                text_align="center",
            ),
            bgcolor=self.dock_color,
            border_radius=dock_width // 2,
            width=dock_width,
            height=20,
            alignment=alignment.center,
        )
        if position == "top_right":
            dock.bottom = 26
            dock.left = 22
        elif position == "center_left":
            dock.top = 15
            dock.left = 22
        elif position == "bottom_left":
            dock.top = 26
            dock.left = 22
        elif position == "bottom_right":
            dock.top = 26
            dock.right = 50
        else:
            dock.bottom = 26
            dock.right = 50

        return dock


def main(page: Page):
    page.vertical_alignment = "center"
    page.horizontal_alignment = "center"
    page.add(
        Dock(position="top_left"),
        Dock(cupertino_icons.CART, position="center_left", count=105, max_value=99),
        Dock(count=15, dock_color="purple500", position="top_left", max_value=9),
        Dock(count=105, dock_color="blue", position="top_right", max_value=99),
        Dock(count=1005, position="bottom_right", max_value=999),
    )


if __name__ == "__main__":
    app(target=main)



from flet import *


class TimelineDot(Container):
    def __init__(
        self,
        icon: str = None,
        icon_size: int = 0,
        icon_color: str = colors.WHITE,
        dot_size: int = 5,
        dot_border_radius: int = 50,
        border: Border = border.all(3, color="white"),
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.content = (
            Icon(
                icon,
                size=icon_size,
                color=icon_color,
            )
            if icon != None
            else None
        )
        self.alignment = alignment.center
        self.padding = padding.all(dot_size)
        self.border = border
        self.border_radius = dot_border_radius


class TimelineSeparator(Container):
    def __init__(
        self, color: str = colors.WHITE, width: int = 2, height: int = 50, **kwargs
    ):
        super().__init__(**kwargs)
        self.content = Container(
            width=width,
            height=height,
            bgcolor=color,
            alignment=alignment.center,
        )


class TimelineEvent(Container):
    def __init__(
        self,
        title: str,
        dot_props: dict = {},
        separator_props: dict = {},
        title_style: TextStyle = None,
        description: str = None,
        description_style: TextStyle = None,
        timestamp: str = None,
        timestamp_style: TextStyle = None,
        content_position: int = 8,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.title = title
        self.description = description
        self.timestamp = timestamp

        self.content_position = content_position
        self.title_style = title_style
        self.description_style = description_style
        self.timestamp_style = timestamp_style

        self.dot_props = dot_props
        self.separator_props = separator_props

        self.content = Row(
            [
                Column(
                    [
                        TimelineDot(**self.dot_props),
                        TimelineSeparator(**self.separator_props),
                    ],
                    alignment="center",
                    horizontal_alignment="center",
                    spacing=0,
                ),
                Container(
                    Column(
                        [
                            Text(
                                self.title,
                                weight=FontWeight.BOLD,
                                size=16,
                                style=self.title_style,
                            ),
                            Text(
                                self.description, size=14, style=self.description_style
                            ),
                            Text(
                                self.timestamp,
                                size=12,
                                color=colors.GREY,
                                style=self.timestamp_style,
                            ),
                            Container(
                                height=self.content_position
                            ),  # A container to adjust the content position
                        ],
                        alignment="start",
                        spacing=5,
                    ),
                    padding=padding.only(0, top=0),
                    margin=margin.all(0),
                ),
            ],
            alignment="start",
            spacing=10,
        )

        self.margin = margin.only(top=10)


class Timeline(Container):
    def __init__(self, events: list, **kwargs):
        super().__init__(**kwargs)
        self.events = events

        self.content = Column(
            [TimelineEvent(**event) for index, event in enumerate(self.events)],
            alignment="start",
            spacing=0,
        )


def main(page: Page):
    page.vertical_alignment = "start"
    page.horizontal_alignment = "center"
    page.padding = 20

    events = [
        {
            "title": "Event 1",
            "description": "This is the description for event 1.",
            "timestamp": "2023-01-01 10:00 AM",
            "dot_props": {
                # "icon": icons.EVENT,
                # "icon_size": 30,
                # "border_color": "red",
                # "border_radius": 2,
                # "dot_border_radius": 0,
                "border": border.all(2, color="blue"),
            },
            "content_position": 0,
            "separator_props": {
                "color": colors.GREEN_500,
                # "width": 3,
                "height": 70,
                "margin": margin.only(top=5),
            },
            "content_position": 25,
        },
        {
            "title": "Event 2",
            "description": "This is the description for event 2.",
            "timestamp": "2023-02-01 12:00 PM",
            # "icon": icons.CALENDAR_TODAY,
            # "icon_size": 25,
            "dot_props": {
                # "border_color": "red",
                # "border_radius": 2,
                # "dot_border_radius": 0,
                "border": border.all(2, color="red"),
            },
            "content_position": 0,
            "separator_props": {
                # "color": colors.BLUE,
                # "width": 3,
                "height": 70,
                "margin": margin.only(top=5),
            },
            "content_position": 25,
        },
        {
            "title": "Event 3",
            "description": "This is the description for event 3.",
            "timestamp": "2023-03-01 02:00 PM",
            # "icon": icons.DATE_RANGE,
            # "icon_size": 20,
            "dot_props": {
                # "border_radius": 2,
                # "dot_border_radius": 0,
                "border": border.all(2, color="green"),
            },
            "content_position": 0,
            "separator_props": {
                "color": colors.BLUE_200,
                # "width": 3,
                "height": 70,
                "margin": margin.only(top=5),
            },
            "content_position": 20,
        },
    ]

    timeline = Timeline(events)

    page.add(timeline)


if __name__ == "__main__":
    app(target=main)



from flet import *


class ShadowPosition:
    TOP_RIGHT: tuple = (5, -5)
    TOP_LEFT: tuple = (-5, -5)
    BOTTOM_RIGHT: tuple = (5, 5)
    BOTTOM_LEFT: tuple = (-5, 5)


class FlatContainer(Container):
    def __init__(self, height: int, **kwargs):
        super().__init__(
            bgcolor=colors.GREY_200,
            height=height,
            alignment=alignment.center,
            border_radius=10,
            shadow=[
                BoxShadow(
                    spread_radius=1,
                    blur_radius=6,
                    color=colors.with_opacity(0.2, colors.BLACK),
                    offset=Offset(2, 2),
                ),
            ],
            **kwargs,
        )


class ConvexContainer(Container):
    def __init__(
        self,
        height: int,
        elevation: float = 0.0,
        shadow1_color: str = "white",
        shadow2_color: str = "black",
        border_radius=10,
        **kwargs,
    ):
        super().__init__(
            height=height,
            alignment=alignment.center,
            border_radius=border_radius,
            gradient=LinearGradient(
                colors=["#cacaca", "#f0f0f0"],
                end=alignment.top_left,
                begin=alignment.center_right,
            ),
            shadow=[
                BoxShadow(
                    spread_radius=1,
                    blur_radius=10,
                    color=colors.with_opacity(elevation, shadow1_color),
                    offset=Offset(-1, -1),
                ),
                BoxShadow(
                    spread_radius=1,
                    blur_radius=10,
                    color=colors.with_opacity(elevation / 2, shadow2_color),
                    offset=Offset(1, 1),
                ),
            ],
            **kwargs,
        )


class FloatingContainer(Container):
    def __init__(
        self,
        height: int,
        border_radius=10,
        bgcolor: str = colors.GREY_200,
        shadow1_color: str = colors.WHITE,
        shadow2_color: str = colors.BLACK,
        shadow_position: ShadowPosition = ShadowPosition.BOTTOM_RIGHT,
        **kwargs,
    ):
        super().__init__(
            bgcolor=bgcolor,
            height=height,
            alignment=alignment.center,
            border_radius=border_radius,
            shadow=[
                BoxShadow(
                    spread_radius=1,
                    blur_radius=6,
                    color=colors.with_opacity(0.2, shadow1_color),
                    offset=Offset(-shadow_position[0], -shadow_position[1]),
                ),
                BoxShadow(
                    spread_radius=1,
                    blur_radius=6,
                    color=colors.with_opacity(0.2, shadow2_color),
                    offset=Offset(*shadow_position),
                ),
            ],
            **kwargs,
        )


def main(page: Page):
    page.bgcolor = colors.GREY_300
    page.horizontal_alignment = "center"
    page.vertical_alignment = "center"
    page.padding = 20
    page.theme_mode = "light"

    neumorphic_flat_button = FlatContainer("Flat Button", height=50)
    neumorphic_pressed_button = ConvexContainer(
        width=100,
        height=100,
        border_radius=50,
        padding=padding.only(20),
        elevation=0.4,
        content=Text("Convex Container", color="black"),
    )
    neumorphic_floating_container = FloatingContainer(100, 100)
    neumorphic_card = FloatingContainer(
        content=Text("Neumorphic Card, This is a neumorphic card."),
        width=200,
        height=100,
    )

    page.add(
        Column(
            [
                neumorphic_flat_button,
                neumorphic_pressed_button,
                neumorphic_floating_container,
                neumorphic_card,
            ],
            alignment="center",
            spacing=20,
        )
    )


if __name__ == "__main__":
    app(target=main)



import flet as ft
from flet import *


# A component for displaying content on an elevated surface
class Paper(Container):
    def __init__(
        self,
        outlined: bool = False,
        elevation: int = 1,
        alignment=ft.alignment.center,
        **kwargs,
    ) -> None:
        super().__init__(alignment=alignment**kwargs)
        self.elevation: int = elevation
        # Define the shadow parameters based on the distance
        self.shadow_params: dict = {
            0: (0, 0, 0, 0),
            1: (1, 2, 0.1, 1),
            2: (2, 4, 0.15, 2),
            3: (3, 6, 0.2, 3),
            4: (4, 8, 0.25, 4),
            8: (6, 10, 0.3, 8),
            12: (8, 12, 0.35, 12),
            16: (10, 14, 0.4, 16),
            24: (12, 14, 0.4, 16),
        }
        spread_radius, blur_radius, opacity, offset_y = self.shadow_params.get(
            self.elevation, (0, 0, 0, 0)
        )
        # Create the shadow based on the parameters
        shadows = [
            ft.BoxShadow(
                spread_radius=spread_radius,
                blur_radius=blur_radius,
                color=ft.colors.with_opacity(opacity, "black"),
                offset=ft.Offset(0, offset_y),
            )
        ]

        # Create the container with the specified shadow
        if outlined:
            self.border = border.all(width=2, color="#80868B")
        self.shadow = shadows


def main(page: Page):
    page.scroll = "always"
    # page.bgcolor = "white"
    page.theme_mode = "light"
    page.spacing = 50
    page.padding = 0
    page.vertical_alignment = "center"
    page.horizontal_alignment = "center"
    # Test the function with different distances
    distances = [0, 1, 2, 3, 4, 8, 12, 16, 24]
    for dist in distances:
        container = Paper(
            elevation=dist,
            width=200,
            height=50,
            bgcolor="white",
            outlined=True,
            content=ft.Text(f"Elevation {dist}", color="black"),
        )
        container.border_radius = 5
        page.add(container)
        if dist == 24:
            container.margin = margin.only(bottom=50)
        container.update()


if __name__ == "__main__":
    app(target=main)



from flet import *


class CardMedia(Container):
    def __init__(
        self,
        image_src: str,
        image_height: int = 140,
        can_expand: bool = False,
        actions: list = [],
        title: str = "Lizard",
        title_style: TextStyle = None,
        description: str = "",
        long_description: str = None,
        description_style: TextStyle = None,
        action_area: bool = False,
        on_click_action_area: ControlEvent = None,
        width=300,
        **kwargs,
    ):
        super().__init__(**kwargs)

        self.bgcolor = "black"
        self.width = width
        self.border_radius = 10
        self.padding = padding.only(bottom=10)

        self.expanded = False

        self.expanded_container = Container(
            ListView([Text(long_description, style=description_style)], auto_scroll=1),
            visible=False,
            padding=padding.only(10, right=10),
            margin=margin.only(bottom=10),
            # height=200,
        )
        self.image_container = Container(
            Image(
                src=image_src,
                fit="cover",
                height=image_height,
                width=self.width,
            ),
            alignment=alignment.center,
            margin=margin.all(0),
            width=400,
        )
        # A container to contain all the actions
        self.actions_row = Container(
            Row(
                [
                    *(action for action in actions),
                    Row(expand=True),
                    Container(
                        IconButton(icons.ARROW_DROP_DOWN, on_click=self.toggle_expand)
                        if can_expand == True
                        else None
                    ),
                ],
                spacing=0,
                alignment="start",
            ),
            margin=margin.only(top=5, right=10),
        )

        self.content = Column(
            [
                self.image_container,
                Container(
                    Column(
                        [
                            Container(
                                Text(
                                    f"{title}",
                                    size=20,
                                    weight=FontWeight.BOLD,
                                    style=title_style,
                                )
                            ),
                            Text(description, style=description_style),
                        ],
                        spacing=5,
                    ),
                    padding=padding.only(left=10, top=10, right=10),
                ),
                self.actions_row,
                self.expanded_container,
            ],
            spacing=0,
        )
        self.shadow = [
            BoxShadow(
                spread_radius=6,
                blur_radius=10,
                color=colors.with_opacity(0.3, "black"),
                offset=Offset(0, 8),
            )
        ]
        # These events are trigged only if the action_area property True
        self.on_click = on_click_action_area
        self.on_hover = self.action_area if action_area else None

    # when the dropdown iconbutton is clicked
    def toggle_expand(self, e):
        self.expanded = not self.expanded
        self.expanded_container.visible = self.expanded
        e.control.icon = icons.ARROW_DROP_UP if self.expanded else icons.ARROW_DROP_DOWN
        self.update()

    # trigged when the action_area is set to True & the Card is hovered by a cursor
    def action_area(self, e):
        e.control.opacity = 0.7 if e.data == "true" else 1.0
        e.control.update()


def main(page: Page):

    page.add(
        CardMedia(
            image_src="https://th.bing.com/th/id/R.5e510c21c45cefceb127a2280c789b72?rik=2ddR7LkLmjAIFA&pid=ImgRaw&r=0",
            description="Lizards are a widespread group of squamate reptiles, with over 6,000 species, ranging across all continents except Antarctica",
            actions=[TextButton("SHARE"), TextButton("LEARN MORE")],
            can_expand=True,
            # action_area=True,
            long_description="""To make the container expand when the drop-down icon is clicked, you can add a callback function that toggles the expansion of the description. This involves updating the visibility and possibly the size of the container when the icon is clicked. Here's how you can implement this:

Define a method to handle the expansion.
Add state to keep track of whether the card is expanded.
Update the icon button to toggle the expansion state.
""",
        )
    )


if __name__ == "__main__":
    app(target=main)



from flet import *
from dataclasses import dataclass


@dataclass
class RatingSize:
    small = 0.7
    medium = 1
    large = 1.5
    extralarge = 2


@dataclass
class RatingType:
    CONTROLLED = "controlled"
    READONLY = "readonly"
    DISABLED = "disabled"


class Rating(Row):
    def __init__(
        self,
        rating_type: str = "controlled",
        max_value: int = 5,
        rating_value: float = 0.0,
        rating_icon: str = icons.STAR_OUTLINE_OUTLINED,
        size: str = "large",
        color: str = colors.WHITE,
        selection_color: str = colors.ORANGE,
        selection_icon: str = icons.STAR,
        half_icon: str = icons.STAR_HALF,
        on_hover_color: str = colors.ORANGE,
    ):
        super().__init__()

        self.max_value: int = max_value
        self.rating_value: float = rating_value
        self.rating_icon: str = rating_icon
        self.selection_icon: str = selection_icon
        self.size = size
        self.selection_color: str = selection_color
        self.color: str = color
        self.on_hover_color: str = on_hover_color
        self.rating_type: str = rating_type

        self.controls = [
            Container(
                IconButton(
                    icon=self.rating_icon,
                    icon_color=self.color,
                    selected_icon_color=self.selection_color,
                    selected_icon=self.selection_icon,
                    data=True,
                    on_click=(
                        self.on_icon_clicked
                        if self.rating_type == RatingType.CONTROLLED
                        else None
                    ),
                    style=ButtonStyle(
                        color={
                            MaterialState.HOVERED: (
                                self.on_hover_color
                                if self.rating_type == RatingType.CONTROLLED
                                else None
                            ),
                        },
                        overlay_color={
                            MaterialState.HOVERED: colors.TRANSPARENT,
                            MaterialState.FOCUSED: colors.WHITE,
                        },
                        padding=padding.all(0),
                    ),
                    scale=1.0,
                    mouse_cursor=(
                        MouseCursor.CLICK
                        if self.rating_type == RatingType.CONTROLLED
                        else MouseCursor.ALIAS
                    ),
                ),
                margin=margin.all(-7),
                on_hover=(
                    self.icon_hovered
                    if self.rating_type == RatingType.CONTROLLED
                    else None
                ),
            )
            for _ in range(self.max_value)
        ]
        # rating size
        if size == "small":
            self.scale = RatingSize.small
        elif size == "medium":
            self.scale = RatingSize.medium
        elif size == "extralarge":
            self.scale = RatingSize.extralarge
        else:
            self.scale = RatingSize.large

        self.spacing = 0
        self.alignment = "center"
        self.opacity = 0.4 if self.rating_type == RatingType.DISABLED else 1

        # implement
        if self.rating_value != 0 and (
            self.rating_type == RatingType.READONLY
            or self.rating_type == RatingType.DISABLED
        ):
            if str(self.rating_value).isdigit() == False:
                for i in range(int(self.rating_value) + 1):
                    self.controls[i].content.selected = True
                    if str(self.rating_value).isdigit() == False:

                        self.controls[
                            (int(self.rating_value))
                        ].content.selected_icon = half_icon
            else:
                for i in range(int(self.rating_value)):
                    self.controls[i].content.selected = True

    def icon_hovered(self, e):

        e.control.content.scale = 1.5 if e.data == "true" else 1.0
        e.control.content.icon_color = (
            self.on_hover_color if e.data == "true" else self.color
        )
        e.control.content.update()

    def on_icon_clicked(self, e):
        rating_list = e.control.parent.parent.controls
        index = rating_list.index(e.control.parent)
        for i in range(len(rating_list)):
            rating_list[i].content.selected = i <= index
            rating_list[i].update()

    # def get_rating_value(self):
    #     if self.size==RatingType.CONTROLLED:
    #         return


def main(page: Page):
    page.vertical_alignment = "center"
    page.alignment = "center"
    page.add(
        # Rating(max_value=5, size="small"),
        Rating(
            rating_icon=icons.STAR_OUTLINE_OUTLINED,
            selection_icon=icons.STAR,
            max_value=5,
            rating_type=RatingType.CONTROLLED,
        ),
        Rating(
            max_value=5,
            selection_icon=icons.STAR,
            rating_value=2.5,
            rating_type=RatingType.READONLY,
            size="large",
        ),
        Rating(
            selection_icon=icons.HEART_FILL,
            half_icon=icons.HEART_BROKEN_OUTLINED,
            max_value=5,
            rating_value=1.5,
            rating_type=RatingType.DISABLED,
        ),
    )


if __name__ == "__main__":
    app(target=main)
